DOS2   ; dos2/asm
 *TITLE	'DOS+II interrupt services'
 ;
 ; externals
 ;	EXTRN	TRADD
 ;TRADD	EQU	$	;PC at NMI time (for trace)
 ;RTCTBL	EQU	$	;real time clock service table
 ;DEBUG	EQU	$	;debug entry point
 ;
 *EJECT
 ;
 ; the following is the NMI service vectored at 0066H
 ;
 NMICODE	EX	(SP),HL	;get address for trace
 	LD	(TRADD),HL ;save it for later
 	EX	(SP),HL ;back the way it was
 	PUSH	HL
 	PUSH	AF
 	PUSH	DE
 	PUSH	BC
 	PUSH	IY
 	PUSH	IX
 	CALL	TIMER	;bump clock counter
 	LD	IY,RTCTBL ;table of RTC tasks
 	LD	A,16	;16 entries in table
 NMILOOP	PUSH	AF	;save counter
 	PUSH	IY
 	LD	HL,NMINEXT ;setup return for sub
 	PUSH	HL	;by placing on the stack
 	LD	A,(IY)	;check for empty slot
 	OR	(IY+1)	;0000H address = nil entry
 	RET	Z	;nil task
 	DEC	(IY+2)  ;see if time to do yet
 	RET	NZ	;next one if not time
 	LD	A,(IY+3) ;else reset the counter
 	LD	(IY+2),A ;from the last byte
 	LD	L,(IY)   ;get address of routine
 	LD	H,(IY+1) ;ABCDEHL free for routine
 	JP	(HL)	;execute subroutine
 NMINEXT LD	DE,4	;4 byte table
 	POP	IY
 	ADD	IY,DE	;point to next entry
 	POP	AF	;get counter
 	DEC	A	;less last loop
 	JR	NZ,NMILOOP ;go to it
 	IN	A,(0FEH) ;clear RTC
 	POP	IX
 	POP	IY	;restore the stack
 	POP	BC
 	POP	DE
 	LD	HL,FLAGK1 ;see if debug is to go
 	BIT	1,(HL)	;set if yes
 	JR	NZ,SETDBG ;go
 	POP	AF
 	POP	HL
 	RETN		;NMI service complete
 SETDBG	POP	AF
 	RES	1,(HL)	;turn off trigger
 	LD	HL,DEBUG ;debug vector
 	EX	(SP),HL	;put on stack, restore HL
 	RETN		;go debug
 *EJECT
 ;
 ; the following are routines to add and remove
 ;	tasks from the interrupt chain
 ;	A = SVC number (59=add, 60=kill)
 ;	C = slot number (0-15)
 ;	DE = vector to subroutine address
 ;		(DE not needed on KILTSK)
 ;	B = countdown time (not needed on KILTSK)
 ; 	Z = successful  NZ = error in A register
 ;
 ; when adding a vector, if C = FF, scan for any
 ;	available slots
 ;	if Z flag set, then C = slot number used
 ;
 ADDNMI	LD	A,C	;get slot number
 	INC	A	;check for FF
 	PUSH	HL	;save needed registers
 	PUSH	BC
 	JR	Z,ADDSCN ;scan for a slot
 	CP	17	;check for max
 	JR	NC,NMIERR ;illegal parameter
 	LD	L,C	;give routine number to HL
 	LD	H,0	;HL = slot number
 	ADD	HL,HL	;times 2
 	ADD	HL,HL	;times 4 (four bytes / entry)
 	LD	BC,RTCTBL ;start of table
 	ADD	HL,BC	;HL => slot entry
 	POP	BC	;restore time parameter
 ADSBAK	CALL	NMIOFF	;turn off NMI
 	LD	(HL),E	;put vector address here
 	INC	HL
 	LD	(HL),D	;MSB of vector
 	INC	HL
 	LD	(HL),B	;put time count here
 	INC	HL
 	LD	(HL),B	;put initial count
 	CALL	NMION	;turn back on now
 	POP	HL	;restore stack
 	XOR	A	;set Z flag for OK
 	RET		;all set
 ADDFND	LD	A,C	;get slot number
 	POP	BC	;restore original
 	LD	C,A	;update slot #
 	DEC	HL	;point to start
 	JR	ADSBAK	;continue with add
 ADDSCN	LD	HL,RTCTBL ;table of tasks
 	LD	BC,0010H ;C = slot, B = counter
 ADDSLP	LD	A,(HL)	;get a byte
 	INC	HL	;point to next
 	OR	(HL)	;see if nil
 	JR	Z,ADDFND ;found a slot
 	INC	HL	;4 byte table
 	INC	HL
 	INC	HL
 	DJNZ	ADDSLP	;go more
 	LD	A,30	;"no more extents" (16 max)"
 NMIRET	POP	BC	;restore stack
 	POP	HL
 	OR	A	;set NZ flag
 	RET
 NMIERR	LD	A,3	;parameter error
 	JR	NMIRET
 ;
 ; when deleting a task, the slot number must be
 ;	explicitly stated in the C register
 ;
 KILNMI	PUSH	DE
 	LD	DE,00 	;nill vector
 	CALL	ADDNMI	;remove entry from table
 	POP	DE	;restore stack
 	RET
 ;
 ; following will turn the NMI service ON and OFF
 ;
 NMIOFF	PUSH	HL	;save it
 	LD	HL,(66H) ;get current vector
 	LD	(NMIVECT),HL ;save the vector
 	LD	HL,45EDH ;RETN opcode
 	LD	(66H),HL ;NMI now disabled
 	POP	HL
 	RET
 ;
 NMION	PUSH	HL	;save it
 	LD	HL,0	;get old vector
 NMIVECT	EQU	$-2	;setup label
 	LD	(66H),HL ;re-install vector
 	POP	HL
 	RET		;NMI now enabled
 ;
 *EJECT
 ;
 ; following is the routine to advance the clock
 ;
 TIMER	LD	HL,TIME$ ;point to current time
 	LD	DE,TIMETBL ;point to max values
 	LD	B,4	;4 bytes for time
 CLOCK1	INC	(HL)	;bump the ticks
 	LD	A,(DE)	;get max
 	SUB	(HL)	;test it
 	RET	NZ	;not time to change yet
 	LD	(HL),A	;reset the value to 0
 	INC	L	;bump time pointer
 	INC	E	;bump max pointer
 	DJNZ	CLOCK1	;do all 4 time values
 ;
 ; optional date adjust for each 24 hour period
 ; leap years are automatically adjusted
 ;
 	PUSH	HL	;save day pointer
 	INC	L	;point to month
 	LD	A,(HL)	;get it
 	DEC	A	;make 1-12 into 0-11
 	ADD	A,E
 	LD	E,A	;(DE) = max days this month
 	INC	L	;point to year
 	LD	A,(HL)	;get the year
 	AND	3	;check for leap year
 	LD	A,28	;28 days hath february
 	JR	NZ,CLOCK3 ;go if not leap year
 	INC	A	;else 29 days this year
 CLOCK3	LD	(MONTHS+1),A ;set # days in FEB
 	POP	HL	;now pointing to day
 	INC	(HL)	;bump the day
 	LD	A,(DE)	;get the max this month
 	SUB	(HL)	;compare to current #
 	RET	NC	;not end of month yet
 	LD	(HL),1	;set first day
 	INC	L	;point to month
 	INC	(HL)	;bump it
 	LD	A,(HL)	;get the month
 	CP	13	;at end of year yet?
 	RET	C	;nope, done
 	LD	(HL),1	;set month 1
 	INC	L	;point to year
 	INC	(HL)	;bump the year
 	RET		;completed
 *EJECT
 ;
 	END
