DOS3A  ; dos3a/asm
 *TITLE	'DOS+II keyboard drivers'
 ;
 ; equates
 KIDCB	EQU	0	;keyboard DCB number
 ;
 ; outside support
 ;	EXTRN	LOCDCB,DELAY,XLATE
 ;LOCDCB	EQU	$	;load IX with DCB address
 ;FLAGK1	EQU	$	;flag #1
 ;XLATE	EQU	$	;translate A from HL/B table
 ;
 *EJECT
 ; the following code is the keyboard driver
 ;	which will normally run under foreground
 ;	control.  this is the normal vector of
 ;	the driver address in the *KI DCB
 ; on entry, C and Z are set to indicate the call type
 ;	IX => DCB
 ; on exit, B contains the input character from the
 ;	keyboard.  The Z flag will return the status
 ;	of the operation.  If NZ, A will contain the
 ;	error number.  If a call is made via @CTL
 ;	then service will wait for a key.
 ;	If call is from @GET, then no waiting
 ;
 KBDCODE	RES	6,(IX+5) ;set non-control
 	JR	C,KBDGET ;request from @GET call
 	JR	Z,KBDERR ;request from @PUT call, error
 	SET	6,(IX+5) ;request from @CTL call
 KBDGET	LD	A,(IX+9) ;get char count
 	OR	A	;set flags on count
 	JR	Z,KBDEMP ;no chars available, see if @CTL
 	LD	H,(IX+7) ;get MSB buffer pointer
 	LD	A,(IX+11) ;take offset
 	PUSH	AF	;save for pointer increment
 	ADD	A,(IX+6) ;LSB buffer
 	LD	L,A	;put it here
 	JR	NC,KI1	;cross page boundary?
 	INC	H
 KI1	LD	B,(HL)	;get character from buffer
 	POP	AF	;get offset
 	CALL	KBDADV	;advance buffer offset
 	LD	(IX+11),A ;update in DCB
 	DEC	(IX+9)	;remove key from counter
 	XOR	A	;load with 0 for no error
 	BIT	7,(IX+5) ;see if translation on
 	RET	Z	;nope, return with Z flag
 	LD	L,(IX+12) ;get translation table ptr
 	LD	H,(IX+13)
 	LD	A,B	;A = char
 	LD	B,(IX+14) ;# entries
 	CALL	XLATE	;translate from table
 	LD	B,A	;save the key
 	XOR	A	;set Z flag (no error)
 	RET		;done, B = key, Z = no error
 KBDEMP	BIT	6,(IX+5) ;@CTRL call?
 	JR	NZ,KBDGET ;wait for a key if yes
 	LD	A,2	;char not available
 	OR	A
 	RET		;NZ = no char
 KBDERR	LD	A,39	;"illegal I/O attempt" error
 	OR	A	;set NZ flag
 	RET		;return in error
 KBDADV	INC	A	;bump buffer offset
 	CP	(IX+8)	;test for max size
 	RET	C	;in boundary
 	XOR	A	;else restart at beginning
 	RET
 *EJECT
 ;
 ; the following is the keyboard driver for the
 ;	vectored interrupt service
 ;
 KBDTASK	PUSH	AF	;save registers
 	PUSH	DE
 	PUSH	HL
 	PUSH	IX
 	LD	A,KIDCB ;keyboard DCB number
 	CALL	LOCDCB	;load IX with DCB address
 	IN	A,(0FCH) ;read the key
 	LD	E,A	;save the key
 	LD	A,(IX+9) ;chars in type
 	CP	(IX+8)	;test for max size
 	JR	NC,KBDOVER ;overflow of buffer
 	LD	H,(IX+7) ;MSB buffer
 	LD	A,(IX+10) ;add offset
 	PUSH	AF	;save for increment
 	ADD	A,(IX+6) ;add to LSB buffer
 	LD	L,A	;save here
 	JR	NC,KBD2	;cross page boundary?
 	INC	H
 KBD2	LD	(HL),E	;put char in buffer
 	INC	(IX+9)	;bump char count in buffer
 	POP	AF	;get offset
 	CALL	KBDADV	;advance pointer
 	LD	(IX+10),A ;update the pointer
 	LD	A,E	;get char
 	LD	HL,FLAGK1 ;flag byte #1
 	CP	3	;BREAK?
 	JR	NZ,KBD3 ;go if not
 	SET	6,(HL)	;set BREAK in buffer
 KBD3	OR	A	;HOLD?
 	JR	NZ,KBDRET ;go if not
 	SET	4,(HL)	;set HOLD in buffer
 KBDRET	POP	IX	;restore stack
 	POP	HL
 	POP	DE
 	POP	AF
 	RETI		;character now in buffer
 KBDOVER SET	1,(IX+5) ;set overflow flag
 	JR	KBDRET	;done
 *EJECT
 ;
 	END
