DOS3B  ; dos3b/asm
 *TITLE	'DOS+II printer drivers'
 ;
 ; equates
 ;
 PRDCB	EQU	2	;printer DCB number
 PRMAX	EQU	4	;max chars output per interrupt
 PRGRPH	EQU	'#'	;convert graphics to this
 ;
 ; external support
 ;
 ;	EXTRN	LOCDCB,BRCV,BTX,XLATE,UCASE,LOOKUP
 ;LOCDCB	EQU	$	;locate DCB
 ;BRCV	EQU	$	;receive char channel B
 ;BTX	EQU	$	;transmit char channel B
 ;XLATE	EQU	$	;translate A from HL/B
 ;UCASE	EQU	$	;convert A to upper case
 ;LOOKUP	EQU	$	;lookup B in table
 ;NMIOFF	EQU	$	;disable NMI
 ;NMION	EQU	$	;enable NMI
 ;
 *EJECT
 ;
 ; increment ring buffer offset, DE = current offset
 ;
 PRINC	INC	DE	;bump offset pointer
 	LD	L,(IX+13) ;get length of buffer
 	LD	H,(IX+14)
 	DEC	HL	;adjust for compare
 	OR	A	;clear carry
 	SBC	HL,DE	;compare
 	RET	NC	;not at end
 	LD	DE,0	;else restart it
 	RET
 ;
 ; fetch printer status
 ;	return Z = ready, NZ = not ready
 ;
 PRIN	BIT	7,(IX+5) ;parallel/serial
 	JR	NZ,PRINS ;get serial status
 	IN	A,(0E0H) ;read parallel status
 	AND	0F0H	;top 4 bits only
 	CP	30H	;7,6 reset, 5,4 set
 	RET		;Z = ready
 PRINS	PUSH	BC	;save from status
 	CALL	BRCV	;get channel B status
 	JR	NZ,PRINSR ;not ready
 	LD	A,13H	;mask
 	CP	B	;ready for output?
 	LD	A,B	;get status
 PRINSR	POP	BC	;restore
 	RET		;Z = ready
 ;
 ; send char to printer (interrupt service)
 ;
 PROUT	BIT	7,(IX+5) ;parallel/serial
 	JR	NZ,PROUTS ;go serial
 	OUT	(0E1H),A ;send parallel
 	RET
 PROUTS	PUSH	BC	;save it
 	LD	B,A	;character to send
 	CALL	BTX	;send to channel B
 	POP	BC
 	RET
 *EJECT
 ;
 ; driver address for printer (SVC 18)
 ;
 PRTCODE	JR	C,PRIN	;input request (status)
 	JR	NZ,PRIN ;control request
 	LD	A,B	;get character
 	CP	0BH	;top of form?
 	JR	Z,PR5	;go if yes
 	CP	0CH	;TOF?
 	JR	Z,PR5
 PR3	CP	80H	;graphics?
 	JR	C,PR1	;skip if yes
 	BIT	4,(IX+5) ;graphic enabled?
 	JR	NZ,PR1	;go if yes
 	LD	A,PRGRPH ;adjust to non-graphic character
 PR1	BIT	3,(IX+5) ;lower case enabled?
 	JR	NZ,PR1X	;go if yes
 	CALL	UCASE	;make it upper case
 PR1X	CP	13	;C/R
 	JR	NZ,PR8	;go if not
 	INC	(IX+10) ;any chars in buffer
 	DEC	(IX+10)
 	JR	NZ,PR8	;go if yes
 	BIT	2,(IX+5) ;transparent mode?
 	JR	NZ,PR8	;go if yes
 PR2	LD	A,10	;else convert to LF
 PR8	CALL	PR11	;output the char
 	CP	10	;LF
 	JR	Z,PR6	;go if yes
 	CP	0BH	;FF?
 	JR	Z,PR7
 	CP	0CH
 	JR	Z,PR7
 	CP	13	;CR
 	JR	NZ,PR14	;go if not
 	BIT	5,(IX+5) ;issue LF after?
 	JR	NZ,PR2	;go if yes
 	JR	PR6	;else skip
 PR14	INC	(IX+10) ;bump char counter
 	LD	A,(IX+9) ;get max chars/line
 	OR	A	;nil?
 	JR	Z,PR13	;go if yes
 	CP	(IX+10) ;see if end of line
 	JR	NZ,PR13	;go if not
 	LD	A,13	;else send CR
 	JR	PR3	;send it for new line
 PR6	LD	(IX+10),0 ;reset char counter
 	INC	(IX+8)	;bump line counter
 	LD	A,(IX+6) ;lines to print/page
 	CP	(IX+8)	;check with line counter
 	JR	NZ,PR13	;go if not at end
 PR5	BIT	1,(IX+5) ;real FF?
 	JR	NZ,PR8	;go if yes
 	LD	A,(IX+7) ;get page length
 	SUB	(IX+8)	;less lines printed
 	LD	C,A	;counter
 	JR	Z,PR7	;go if not time
 PR9	LD	A,10	;LF
 	CALL	PR11	;send it
 	DEC	C	;more?
 	JR	NZ,PR9	;go if yes
 PR7	LD	(IX+8),0 ;clear the line counter
 	LD	(IX+10),0 ;clear char counter
 PR13	LD	A,B	;get char back
 	RET
 PR11	EX	AF,AF'	;AF' = CHAR TO PRINT
 PR12	LD	L,(IX+13)
 	LD	H,(IX+14) ;get max buffer size
 	LD	E,(IX+15)
 	LD	D,(IX+16) ;get chars in buffer
 	OR	A	;clear carry
 	SBC	HL,DE	;see if room
 	JR	NC,PR12	;wait for room
 	INC	DE	;bump buff count
 	CALL	NMIOFF	;disable for DCB change
 	LD	(IX+15),E ;update counter
 	LD	(IX+16),D
 	CALL	NMION	;can enable now
 	LD	L,(IX+11) ;get spool buffer
 	LD	H,(IX+12)
 	LD	E,(IX+17) ;get add offset
 	LD	D,(IX+18)
 	ADD	HL,DE	;point to buffer
 	EX	AF,AF'	;get char back
 	CALL	PRXLATE	;see if translation on
 	LD	(HL),A	;put in buffer
 	CALL	PRINC	;increment offset
 	LD	(IX+17),E ;update DCB offset
 	LD	(IX+18),D
 	RET		;done
 PRXLATE	BIT	6,(IX+5) ;translation on?
 	RET	Z	;nope
 	PUSH	HL	;save buff ptr
 	PUSH	BC	;save char
 	LD	L,(IX+21) ;get translation table address
 	LD	H,(IX+22)
 	LD	B,(IX+23) ;translation table length
 	CALL	XLATE	;translate the char
 	POP	BC
 	POP	HL
 	RET		;A = translated char
 *EJECT
 ;
 ; send a line of text to the printer (SVC 19)
 ;
 PRLINE	PUSH	HL	;save
 	PUSH	BC
 	LD	A,B	;see if nil length
 	OR	A
 	JR	Z,PRLRET ;return if nil
 PRLLP	PUSH	BC	;save length
 	LD	B,(HL)	;get char
 	CALL	PRCHAR	;send to printer
 	JR	NZ,PRLBAK ;error, return
 	POP	BC	;get counter
 	INC	HL	;bump buffer
 	DJNZ	PRLLP	;go for length
 	XOR	A	;Z = OK
 PRLRET	CP	C	;last char
 	LD	B,C	;get it
 	CALL	NZ,PRCHAR ;send terminator
 PRLBAK	POP	BC	;restore stack
 	POP	HL
 	RET		;Z = OK, NZ = error
 *EJECT
 ;
 ; control vector for printer services (SVC 95)
 ;
 PRCTRL	PUSH	IX	;save it
 	LD	A,PRDCB	;DCB number
 	CALL	LOCDCB	;find it
 	PUSH	HL	;save till command verified
 	LD	HL,PRCTBL ;table of control vectors
 	CALL	LOOKUP 	;see if valid command
 	JR	NZ,PRCERR ;invalid command
 	PUSH	HL	;save vector
 	LD	HL,PRCRET ;return vector to load regs
 	EX	(SP),HL	;to the stack, HL=vector
 	JP	(HL)	;go vector
 PRCERR	POP	HL	;restore original
 	POP	IX
 	LD	A,3	;parameter error
 	RET		;NZ already set
 PRC1	SET	7,(IX+5) ;set serial
 PRC0	RET		;get status byte
 PRC2	RES	7,(IX+5) ;set parallel
 	RET
 PRC3	LD	(IX+8),C ;reset line counter
 	RET
 PRC4	LD	(IX+10),C ;reset char counter
 	RET
 PRC5	SET	2,(IX+5) ;begin transparent
 	RET
 PRC6	RES	2,(IX+5) ;end transparent
 	RET
 PRC7	SET	3,(IX)	;set NIL (dummy mode)
 	RET
 PRC8	RES	3,(IX)	;reset NIL
 	RET
 PRC9	SET	5,(IX+5) ;set LF on CR
 	RET
 PRC10	RES	5,(IX+5) ;reset LF on CR
 	RET
 PRCRET	POP	HL	;restore stack
 	LD	B,(IX+7) ;page length
 	LD	C,(IX+6) ;max lines
 	LD	D,(IX+9) ;max chars
 	LD	H,(IX+10) ;char count
 	LD	L,(IX+8) ;line count
 	LD	E,'P'	;parallel
 	BIT	7,(IX+5) ;is it?
 	POP	IX	;restore it
 	RET	Z
 	LD	E,'S'	;serial
 	XOR	A	;set Z for OK
 	RET		;done
 ;
 ; lookup table for PRCTRL vectors
 ;
 PRCTBL	DB	0	;get status
 	DW	PRC0
 	DB	1	;select serial
 	DW	PRC1
 	DB	2	;select parallel
 	DW	PRC2
 	DB	3	;reset line counter to C
 	DW	PRC3
 	DB	4	;reset char counter to C
 	DW	PRC4
 	DB	5	;begin transparent
 	DW	PRC5
 	DB	6	;end transparent
 	DW	PRC6
 	DB	7	;begin dummy
 	DW	PRC7
 	DB	8	;end dummy
 	DW	PRC8
 	DB	9	;begin auto LF
 	DW	PRC9
 	DB	10	;end auto LF
 	DW	PRC10
 	DB	-1	;terminator
 *EJECT
 ;
 	END
