DOS3D  ; dos3d/asm
 *TITLE	'DOS+II rs232 drivers'
 ;
 ; equates
 SIODA	EQU	3	;SIO channel A DCB number
 SIODB	EQU	4	;SIO channel B DCB number
 OFFSIOA	EQU	0	;CH-A interrupt service offset
 OFFSIOB	EQU	0	;CH-B interrupt service offset
 SIOLEN	EQU	20	;length of SIO input buffer
 ;
 ; externals
 ;	EXTRN	LOOKUP,LOCDCB,UCASE,XLATE
 ;LOOKUP	EQU	$	;lookup table vector
 ;LOCDCB	EQU	$	;locate DCB
 ;UCASE	EQU	$	;convert A to upper case
 ;XLATE	EQU	$	;translate A from HL/B
 *EJECT
 ;
 ; vector for SIO (in/out/ctrl) (SVC's 96-101)
 ;	on entry, IX=DCB
 ;		 C = input a byte
 ;		 Z = output a byte
 ;		else @CTRL call (wait on input)
 ;	on exit, B = char, A = status
 ;		 Z = OK, char in B
 ;		NZ = no character
 ;		 Carry = carrier lost
 ;
 SIOCODE	PUSH	BC	;save on top
 	LD	C,(IX+8) ;get base port
 	RES	4,(IX+5) ;turn off @CTRL call
 	JR	C,ABRCV	;called via @GET (receive)
 	JR	Z,ABTX	;called via @PUT (send)
 	SET	4,(IX+5) ;called via @CTRL (input wait)
 ABRCV	BIT	7,(IX+5) ;initialized?
 	LD	A,1	;function does not exist
 	JP	NZ,SIORET
 ABRCVWT	XOR	A	;set Z flag (wait loop for @CTRL)
 	OR	(IX+18) ;any chars in buffer?
 	OR	(IX+19)
 	JR	NZ,RXIN	;char in buffer
 	BIT	4,(IX+5) ;see if @CTRL call
 	JR	NZ,ABRCVWT ;wait if yes
 	CALL	RSTATUS	;else get status
 	SCF		;C = error
 	BIT	3,B	;carrier detect?
 	JP	NZ,SIORET ;return if not
 	OR	-1	;set NZ
 	LD	A,B	;restore status, NC
 	JP	SIORET	;done
 RXIN	LD	E,(IX+22) ;take offset
 	LD	D,(IX+23)
 	LD	L,(IX+14) ;get buffer start
 	LD	H,(IX+15)
 	ADD	HL,DE	;get taker pointer
 	LD	A,(HL)	;get the char
 	AND	(IX+18)	;mask for word length
 	LD	B,A	;B = char
 	INC	HL
 	LD	C,(HL)	;C = status byte
 	CALL	SIOINC	;bump buffer
 	LD	(IX+22),E ;update pointer
 	LD	(IX+23),D
 	DI		;disable for decrement
 	LD	E,(IX+18) ;get counter
 	LD	D,(IX+19)
 	DEC	DE
 	LD	(IX+18),E
 	LD	(IX+19),D
 	EI		;can enable now
 	BIT	6,(IX+5) ;translation on for input ?
 	JR	Z,SIO4	;continue if not
 	LD	A,B	;get char
 	LD	L,(IX+24) ;translation table address
 	LD	H,(IX+25)
 	LD	B,(IX+26) ;# entries
 	CALL	XLATE	;translate the char
 	LD	B,A	;put char back
 SIO4	POP	HL	;get old BC
 	LD	A,C	;get status
 	LD	C,L	;original C
 	CP	A	;set Z flag for OK
 	RET		;done
 *EJECT
 ;
 ; SIO transmit services
 ;
 ABTX	BIT	7,(IX+5) ;initialized?
 	LD	A,1	;function does not exist
 	JR	NZ,SIORET
 	LD	A,B	;get character
 	AND	(IX+11)	;word length mask
 	LD	E,A	;save it here
 TXWT	CALL	RSTATUS	;get status byte
 	BIT	2,B	;tx buffer empty?
 	JR	NZ,TXSEND ;send it if yes
 	BIT	3,B	;carrier still detected?
 	JR	NZ,TXWT	;ok if set
 	OR	-1	;set NZ
 	SCF		;C = no carrier
 	LD	A,B	;get status
 	JR	SIORET	;return in error
 TXSEND	LD	C,(IX+8) ;base SIO port
 	DEC	C
 	DEC	C	;point to data port
 	BIT	5,(IX+5) ;translation?
 	JR	Z,SIO4B	;go if not
 	LD	L,(IX+27) ;translation table
 	LD	H,(IX+28)
 	LD	B,(IX+29) ;# entries in table
 	LD	A,E	;get char
 	CALL	XLATE	;translate it
 	LD	E,A	;put it back
 SIO4B	OUT	(C),E	;send the byte
 	XOR	A	;set Z for OK
 	LD	A,B	;get status byte
 SIORET	POP	BC	;restore chars
 	RET		;done
 *EJECT
 ;
 ; SIO control vector CH A (SVC 100)
 ACTRL	LD	A,SIODA ;get DCB number
 	JR	ABCTRL
 ; SIO control vector CH B (SVC 101)
 BCTRL	LD	A,SIODB ;get DCB number
 ABCTRL	PUSH	IX	;save it
 	CALL	LOCDCB	;find the DCB
 	PUSH	HL	;save user parameters
 	PUSH	BC
 	LD	HL,SIOCTL ;table of @CTRL service
 	CALL	LOOKUP	;see if entry valid
 	LD	A,3	;invalid command parameter
 	JR	NZ,SIOCRET ;unstack
 	LD	A,(IX+13) ;get CTC R5 control byte
 	LD	C,(IX+8) ;get SIO base port
 	JP	(HL)	;go vector
 SIOC0	CALL	RSTATUS	;get status byte
 SIOCBAK	POP	BC	;get BC
 	LD	B,A	;place result here
 	PUSH	BC	;put it back
 	XOR	A	;set Z flag
 SIOCRET	POP	BC
 	POP	HL
 	POP	IX
 	RET
 SIOC1	LD	A,(IX+18) ;LSB counter
 	JR	SIOCBAK	;return it in B register
 SIOC2	SET	1,A	;turn on request to send
 SIOOK	EX	AF,AF'	;save command
 	LD	A,5	;select R5
 	OUT	(C),A
 	EX	AF,AF'	;get command
 	OUT	(C),A	;send it
 	XOR	A	;set Z flag
 	JR	SIOCRET	;done
 	XOR	A
 	JR	SIORET	;done
 SIOC3	RES	1,A	;turn off request to send
 	JR	SIOOK	;OK
 SIOC4	SET	4,A	;set BREAK
 	JR	SIOOK	;done
 SIOC5	RES	4,A	;reset BREAK
 	JR	SIOOK	;send it to the port
 SIOC6	DI		;must disable for reset
 	XOR	A
 	LD	(IX+18),A ;counter
 	LD	(IX+19),A
 	LD	(IX+20),A ;add offset
 	LD	(IX+21),A
 	LD	(IX+22),A ;take offset
 	LD	(IX+23),A
 	EI		;done
 	JR	SIORET	;return
 SIOC7	DI		;disable for reset
 	LD	A,30H	;reset, enable char interrupt
 	OUT	(C),A
 	LD	A,10H	;reset ext/status interrupts
 	OUT	(C),A
 	LD	A,28H	;reset tx interrupt pending
 	OUT	(C),A
 	LD	A,0C0H	;reset tx underrun
 	OUT	(C),A
 	EI
 	XOR	A	;no error
 	JR	SIORET	;done
 ;
 SIOCTL	DEFB	0	;get status
 	DEFW	SIOC0
 	DEFB	1	;get char count in buffer
 	DEFW	SIOC1
 	DEFB	2	;turn on request to send
 	DEFW	SIOC2
 	DEFB	3	;turn off request to send
 	DEFW	SIOC3
 	DEFB	4	;send break sequence
 	DEFW	SIOC4
 	DEFB	5	;turn off break sequence
 	DEFW	SIOC5
 	DEFB	6	;reset receive buffer to nil
 	DEFW	SIOC6
 	DEFB	7	;reset SIO error condition
 	DEFW	SIOC7
 	DEFB	-1	;terminator
 *EJECT
 ;
 ; fetch the status of the SIO port
 ;	set the mask byte so 0 bits = OK
 ;
 RSTATUS	LD	C,(IX+8) ;get base SIO port
 	LD	A,10H	;clear the latch
 	OUT	(C),A
 	IN	B,(C)	;read R0
 	LD	A,1	;select R1
 	OUT	(C),A
 	IN	C,(C)	;read R1
 	LD	A,B	;get R0
 	AND	0ACH	;break, CTS, TX, DCD
 	XOR	2CH	;correct bits for no error
 	AND	20H	;CTS?
 	JR	Z,SIO3	;go if yes
 	SET	0,B	;RX not avail
 	RES	5,B	;no CTS
 SIO3	LD	A,C	;get R1
 	AND	70H	;CRC, RX overrun, parity
 	OR	B	;complete mask
 	OR	(IX+12)	;mask with interrupt error
 	LD	(IX+12),0 ;clear the mask
 	LD	B,A	;B = resulting status
 	RET
 ;
 ; advance buffer pointer for ring buffer
 ;
 SIOINC	INC	DE	;bump by 2
 	INC	DE
 	PUSH	HL	;save for test
 	LD	L,(IX+16) ;get max length
 	LD	H,(IX+17)
 	DEC	HL
 	DEC	HL	;adjust for compare
 	OR	A	;clear carry
 	SBC	HL,DE	;test for max
 	POP	HL	;restore
 	RET	NC	;not at end
 	LD	DE,0	;else restart it
 	RET
 *EJECT
 ;
 ; SIO interrupt services
 ;
 RSINTA	PUSH	AF	;save everything
 	LD	A,SIODA	;channel A DCB number
 	JR	RSINT
 RSINTB	PUSH	AF
 	LD	A,SIODB	;channel B DCB number
 RSINT	PUSH	BC	;save rest
 	PUSH	DE
 	PUSH	HL	;save more
 	PUSH	IX
 	CALL	LOCDCB	;locate DCB
 	LD	C,(IX+8) ;get base port
 	DEC	C
 	DEC	C	;C = data port
 	IN	B,(C)	;get character
 	LD	L,(IX+18) ;get byte counter
 	LD	H,(IX+19)
 	LD	E,(IX+16) ;get max length
 	LD	D,(IX+17)
 	INC	HL
 	INC	HL	;room for 2 more bytes?
 	OR	A	;clear carry for compare
 	SBC	HL,DE
 	LD	A,20H	;bit 5 overflow mask
 	JR	NC,RSOVER ;overflow
 	LD	E,(IX+20) ;get adder offset
 	LD	D,(IX+21)
 	LD	L,(IX+14) ;get buffer address
 	LD	H,(IX+15)
 	ADD	HL,DE
 	LD	(HL),B	;put char in buffer
 	CALL	SIOINC	;advance pointer
 	LD	(IX+20),E
 	LD	(IX+21),D ;update counter
 	LD	E,(IX+18) ;get counter
 	LD	D,(IX+19)
 	INC	DE
 	LD	(IX+18),E ;update it
 	LD	(IX+19),D
 	CALL	RSTATUS	;get status byte
 	INC	HL
 	LD	(HL),B	;put status byte in
 	XOR	A	;mask for OK
 RSOVER	OR	(IX+12)	;set error mask for foreground
 	LD	(IX+12),A ;update to DCB
 	POP	IX	;restore stack
 	POP	HL
 	POP	DE
 	POP	BC
 	POP	AF
 	RETI		;return from interrupt
 *EJECT
 ;
 	END
