DOS5   ; dos5/asm
 ;*title	'DOS+II - system routines'
 ;
 ; INITIO	SVC 0
 ; initialize video, keyboard, printer
 ;
 INITIO	PUSH	BC	;save registers from params
 	PUSH	DE
 	LD	BC,-1	;B<>0=80 column, C<>0=normal
 	CALL	VDINIT	;initialize video
 	JR	NZ,INIORET ;error
 	CALL	KBINIT	;initialize keyboard
 	JR	NZ,INIORET ;error
 	LD	BC,423CH ;B=page length, C=printed lines
 	LD	D,84H	;D = chars/line
 	CALL	PRINIT	;initialize printer
 INIORET	POP	DE
 	POP	BC
 	RET		;Z = result of inits
 ;
 ; SETUSR	SVC 2
 ; insert/remove vector into SVC table
 ;
 SETUSR	LD	A,B	;get SVC # affected
 	RLCA		;see if >127
 	JR	NC,SETUOK ;go if valid
 	LD	A,3	;parameter error
 	OR	A
 	RET		;NZ = error
 SETUOK	PUSH	DE	;save this
 	EX	DE,HL	;DE = user vector
 	LD	L,A	;L = SVC vector offset
 	LD	H,2	;starts on page 2
 	XOR	A	;set Z
 	CP	C	;Z = remove
 	JR	Z,SETREM ;remove the vector
 	LD	(HL),E	;else add vector
 	INC	HL
 	LD	(HL),D
 SETUDN	EX	DE,HL	;HL = user vector
 	POP	DE	;restore DE
 	XOR	A	;set Z flag for OK
 	RET
 SETREM	DEC	A	;A = FFH
 	LD	E,(HL)	;return to user what it was
 	LD	(HL),A	;set as nil
 	INC	HL
 	LD	D,(HL)
 	LD	(HL),A
 	JR	SETUDN
 ;
 ; SETBRK	SVC 3
 ; insert/remove user BREAK processor
 ;
 SETBRK	PUSH	HL	;save it
 	LD	HL,FLAGK1 ;point to flag byte
 	LD	A,D	;check command
 	OR	E	;zero?
 	JR	Z,REMBRK ;remove break handler
 	LD	(DATAK4),DE ;save vector address
 	SET	7,(HL)	;set BREAK processor active
 	RES	6,(HL)	;reset BREAK pressed
 	XOR	A	;OK
 	JR	SETBRET ;return
 REMBRK	BIT	7,(HL)	;active now?
 	LD	A,1	;function does not exist
 	JR	Z,SETBRET ;return in error
 	LD	DE,(DATAK4) ;get old vector
 	RES	7,(HL)	;turn off processor
 	XOR	A	;set OK
 SETBRET	POP	HL	;restore stack
 	OR	A	;set flags
 	RET		;done
 ;
 ; the following is a translation subroutine that
 ;	can be used for input/output
 ;	the A register contains the char to check
 ;	HL => table of 2 byte entries
 ;	compare byte, translation byte
 ;	the number of entries is in the B register
 ;
 XLATE	CP	(HL)	;this entry match?
 	JR	Z,GOXLATE ;found it, translate
 	INC	HL	;bump table pointer
 	INC	HL	;to next entry
 	DJNZ	XLATE	;go for B register entries
 	RET		;not found, do not change
 GOXLATE INC	HL	;point to xlate byte
 	LD	A,(HL)	;get it
 	RET		;A = translated byte
 ;
 ;
 ; routine to convert char in A to upper case
 ;
 UCASE	CP	60H	;already upper?
 	RET	C	;don't change
 	CP	80H	;non-ascii
 	RET	NC	;don't change
 	AND	5FH	;else make it upper
 	RET
 ;
 ; DELAY		SVC 6
 ; decrement BC till 0
 ;
 DELAY	PUSH	BC	;don't change it
 	DEC	BC	;less 1
 	LD	A,B	;see if any left
 	OR	C
 	JR	NZ,DELAY+1 ;go more if yes
 	POP	BC
 	RET
 ;
 ; VIDKEY	SVC 12
 ; display text string to video and get keyboard string
 ;
 VIDKEY	PUSH	BC	;save char/key count
 	LD	C,0	;nil char at display end
 	CALL	VDLINE	;display line to video
 	POP	BC	;restore it
 	RET	NZ	;error already
 	LD	B,C	;max keys to fetch
 	LD	H,D	;pass buffer to HL
 	LD	L,E
 	CALL	KBLINE	;get string from keyboard
 	POP	HL	;restore string pointer
 	RET
 VIDKEY	PUSH	HL	;save string pointer
 	PUSH	BC	;save char count/key count
 	LD	C,0	;send nil to string end
 	CALL	VDLINE	;display to video
 	JR	Z,VIDK2 ;go if no error
 	POP	HL	;else return with vdline BC
 	POP	HL
 	RET		;return in error
 VIDK2	POP	BC	;restore
 	LD	B,C	;max chars to input
 	LD	H,D	;give buffer ptr to HL
 	LD	L,E
 	CALL	KBLINE	;get line from keyboard
 	POP	HL	;restore HL string pointer
 	RET		;done, Z = status
 ;
 ; RANDOM	SVC 20
