DOSINI ; dosinitk/asm
 ;
 ; RS232 initialization (SVC 55)
 ;
 ; definition of SIO initialization table
 ;	Channel A data
 ;
 SIOA	DEFB	3	;# of tables
 ; setup for SIO ports
 	DEFB	11	;# entries this table
 	DEFB	0F6H	;SIO base port address
 	DEFB	18H	;channel reset
 	DEFB	4	;R4
 	DEFB	44H	;x16, stop=1
 	DEFB	3	;R3
 	DEFB	0C1H	;word=8, RX
 	DEFB	5	;R5
 	DEFB	64H	;word=8, TX
 	DEFB	11H	;reset int, select R1
 	DEFB	0	;disable interrupts
 	DEFB	2	;interrupt vector R2
 	DEFB	OFFSIOA	;offset SIOA interrupt driver
 ; CTC #1 setup
 	DEFB	3	;# entries this table
 	DEFB	0F0H	;CTC-0 base port address
 	DEFB	43H	;count, reset
 	DEFB	7	;timer, count follows
 	DEFB	34H	;default 300 baud
 ; CTC #2 setup
 	DEFB	3
 	DEFB	0F1H
 	DEFB	43H
 	DEFB	7
 	DEFB	34H
 ;
 ;	channel B data
 SIOB	DEFB	2	;# tables
 	DEFB	11
 	DEFB	0F7H	;base port
 	DEFB	18H
 	DEFB	4
 	DEFB	44H
 	DEFB	3
 	DEFB	0C1H
 	DEFB	5
 	DEFB	64H
 	DEFB	11H
 	DEFB	0
 	DEFB	2
 	DEFB	OFFSIOB	;interrupt service offset B
 ; CTC #1
 	DEFB	3
 	DEFB	0F2H	;CTC port address
 	DEFB	43H
 	DEFB	7
 	DEFB	34H
 ; CTC #2 (dummy for channel B)
 	DEFB	0
 	DEFB	0
 	DEFB	0
 	DEFB	0
 	DEFB	0
 ;
 ; baud table
 ;
 SIOBAUD	DEFW	8E07H	;110
 	DEFW	6807H	;150
 	DEFW	3407H	;300
 	DEFW	1A07H	;600
 	DEFW	0D07H	;1200
 	DEFW	3447H	;2400
 	DEFW	1A47H	;4800
 	DEFW	0D47H	;9600
 ;
 ; rs232 intialization
 ;
 RS232C	PUSH	IX	;save needed
 	PUSH	IY
 	PUSH	HL	;HL => user parameter list
 	POP	IY
 	XOR	A	;see if add/delete
 	CP	B
 	EX	AF,AF'	;AF' = Z = REMOVE
 	EXX		;save registers
 	LD	A,(IY)	;get channel #
 	CALL	UCASE	;convert to upper case
 	SUB	'A'	;remove ascii
 	LD	HL,SIOABUF ;SIO buffer channel A
 	LD	IX,SIOA	;parameter list CH-A
 	LD	BC,SIODA<8+0F6H ;B=DCB#, C=base port
 	LD	DE,0F0F1H ;CTC port addresses
 	JR	Z,RSCGO	;have the channel
 	LD	HL,SIOBBUF ;SIO buffer channel B
 	LD	IX,SIOB	;parameter list CH-B
 	INC	B	;bump DCB #
 	INC	C	;bump base port
 	LD	DE,0F200H ;CTC port (00=dummy)
 	DEC	A	;channel B?
 	LD	A,3	;parameter error
 	JR	NZ,RSCRET ;return if illegal channel
 RSCGO	LD	A,B	;get DCB #
 	PUSH	IX	;save config list
 	CALL	LOCDCB	;locate the DCB
 	EX	AF,AF'	;Z = remove channel
 	JR	Z,RSCREM ;remove it
 	LD	(IX+9),D ;CTC ports
 	LD	(IX+10),E
 	POP	DE
 	LD	(IX+6),E ;SIO parameter list
 	LD	(IX+7),D
 	LD	(IX+14),L ;buffer address
 	LD	(IX+15),H
 	LD	(IX+8),C ;base SIO port (status)
 	LD	(IX+16),20 ;buffer length LSB
 	LD	(IX+17),0 ;MSB buffer
 	CALL	SIOPARM	;get user parameters
 	JR	NZ,RSCRET ;return in error
 	CALL	SETSIO	;setup the SIO and CTC ports
 	XOR	A	;set Z flag
 	LD	(IX+18),A ;buffer counter
 	LD	(IX+19),A
 	LD	(IX+20),A ;add offset
 	LD	(IX+21),A
 	LD	(IX+22),A ;take offset
 	LD	(IX+23),A
 	RES	7,(IX+5) ;set channel as initialized
 RSCRET	EXX		;restore registers
 	POP	IY
 	POP	IX
 	RET		;Z = result
 RSCREM	SET	7,(IX+5) ;turn off channel
 	SET	3,(IX)	;set as nil device
 	LD	C,(IX+8) ;get base port
 	LD	A,18H	;channel reset
 	OUT	(C),A
 	LD	A,3	;disable interrupts, reset
 	LD	C,(IX+9) ;CTC #1
 	OUT	(C),A
 	LD	C,(IX+10) ;CTC #2
 	INC	C
 	DEC	C	;see if nil
 	JR	Z,SIO6	;go if nil
 	OUT	(C),A	;else reset CTC
 SIO6	POP	IX	;restore stack
 	XOR	A	;Z = OK
 	JR	RSCRET	;return
 ;
 ; following sends data to SIO and CTC ports
 ;
 SETSIO	LD	L,(IX+6) ;get parameter list address
 	LD	H,(IX+7)
 	LD	E,(HL)	;get table count
 	INC	HL
 SETSLP	LD	B,(HL)	;get data count
 	INC	HL
 	LD	C,(HL)	;get port #
 	INC	HL
 	OTIR		;send block to port
 	DEC	E	;more?
 	JR	NZ,SETSLP ;go if yes
 	RET		;else done
 ;
 ; following interprets data from user parameter list
 ;
 SIOPARM	LD	L,(IX+6) ;get configure parameter list
 	LD	H,(IX+7)
 	PUSH	IY	;user list
 	EX	(SP),HL	;HL => user list
 	POP	IY	;IY => configuration table
 	INC	HL
 	LD	A,(HL)	;get baud
 	OR	A	;must be 1-8
 	JR	Z,RSCERR ;error if 0
 	CP	9
 	JR	NC,RSCERR ;error if >8
 	PUSH	HL
 	LD	HL,SIOBAUD ;baud table
 	DEC	A	;adjust for 0-7
 	SLA	A	;double for table
 	ADD	A,L	;add to LSB
 	LD	L,A
 	JR	NC,SIO7	;page boundary?
 	INC	H	;bump page
 SIO7	LD	A,(HL)	;get counter select
 	INC	HL
 	LD	H,(HL)	;get count
 	LD	(IY+17),A ;put in table
 	LD	(IY+22),H ;count
 	POP	HL
 	INC	HL
 	LD	A,(HL)	;word length
 	CP	5	;must be 5-8
 	JR	C,RSCERR
 	CP	9
 	JR	NC,RSCERR
 	LD	B,A	;for loop
 	XOR	A	;start mask
 SIO8	SCF		;set a bit
 	RLA		;move into A
 	DJNZ	SIO8	;for word length
 	LD	(IX+11),A ;save mask in DCB
 	LD	A,(HL)	;get again
 	SUB	5	;adjust for strange bits
 	JR	Z,SIO9
 	CP	3
 	JR	Z,SIO9
 	XOR	3	;reverse bits
 SIO9	RRCA		;align bits
 	RRCA
 	RRCA
 	LD	B,A	;save temp result
 	OR	8AH	;DTR, TX enable, RTS, length
 	LD	(IY+9),A ;save into TX table
 	LD	(IX+13),A ;save for @CTL calls (R5)
 	RLCA		;align
 	OR	21H	;auto trigger, RX enable
 	LD	(IY+7),A ;put into RX table
 	LD	B,40H	;parity mask
 	INC	HL
 	LD	A,(HL)	;get parity
 	CALL	UCASE	;make it upper case
 	CP	'N'	;none?
 	JR	Z,RSPAR	;have it
 	SET	0,B
 	CP	'O'	;odd?
 	JR	Z,RSPAR
 	SET	1,B
 	CP	'E'	;even?
 	JR	NZ,RSCERR ;parameter error
 RSPAR	INC	HL
 	LD	A,(HL)	;stop bits
 	CP	4	;must be 0-3
 	JR	NC,RSCERR ;error if not
 	SLA	A	;align
 	SLA	A
 	OR	B	;merge with mask
 	LD	(IY+5),A ;put into parameter table
 	XOR	A	;set Z for OK
 	RET		;parameters OK
 RSCERR	LD	A,3	;parameter error
 	OR	A	;set NZ
 	RET
 ;
 ;
 ; printer initialization (SVC 17)
 ;
 PRINIT	PUSH	IX	;need it
 	PUSH	DE	;need E for mask
 	LD	A,PRDCB	;DCB number for printer
 	CALL	LOCDCB	;locate the DCB
 	XOR	A
 	CP	B	;page length
 	JR	Z,PRIN1	;check both B&C = 0
 	CP	C	;printed lines
 	JR	NZ,PRIN1 ;OK
 	LD	A,B	;length
 	CP	C	;printed
 	JR	C,PRERR	;invalid
 	LD	E,0	;mask
 	JR	PRIN2	;go
 PRIN1	OR	B	;both must be 0
 	OR	C
 	JR	NZ,PRERR ;parameter error
 	LD	E,2	;real FF
 PRIN2	LD	A,(IX+5) ;get control byte
 	AND	0F8H	;low 3 bits off
 	OR	E	;set form feed control
 	INC	D	;see if 0 max chars
 	DEC	D
 	JR	NZ,PRIN3 ;go if <>0
 	OR	1	;set real tabs
 PRIN3	LD	(IX+5),A ;update control byte
 	RES	3,(IX)	;turn off nil device
 	XOR	A	;set Z flag for OK
 	LD	(IX+6),C ;printed lines/page
 	LD	(IX+7),B ;page length
 	LD	(IX+8),A ;line counter
 	LD	(IX+9),D ;max chars/line
 	LD	(IX+10),A ;char counter
 	JR	PRIN4	;done
 PRERR	LD	A,3	;set parameter error
 	OR	A	;set NZ
 PRIN4	POP	DE	;restore stack
 	POP	IX
 	RET		;Z = OK
 ;
 ; keyboard initialization (SVC 1)
 ;
 KBDINIT	LD	A,KIDCB	;DCB number for keyboard
 	PUSH	IX	;save it
 	CALL	LOCDCB	;load IX with DCB
 	LD	(IX+8),80 ;default length of buffer
 	LD	HL,KBDBUFF ;keyboard buffer address
 	LD	(IX+6),L ;put LSB into DCB
 	LD	(IX+7),H ;put in MSB
 	IN	A,(0FCH) ;read the keyboard port
 	LD	BC,80H	;make sure its not hardware
 	CALL	DELAY	;decrement BC till 0
 	IN	A,(0FCH) ;make sure it's clear
 	LD	A,(IX+5) ;get flags
 	AND	80H	;save xlate only
 	LD	(IX+5),A ;put it back
 	XOR	A	;set Z flag
 	LD	(IX+9),A ;chars in buffer
 	LD	(IX+10),A ;adder offset
 	LD	(IX+11),A ;taker offset
 	PUSH	HL
 	LD	HL,FLAGK1 ;flag byte #1
 	LD	A,(HL)	;get flag
 	AND	0FH	;turn off BREAK/HOLD
 	LD	(HL),A	;update flag
 	POP	HL
 	POP	IX
 	RET		;done
 ;
 	END
