DOSRTC ; dosrtck/asm
 ;
 ; real time clock service routines
 ; IX, IY, HL, DE, BC, AF already saved
 ;
 ; following is the real time clock printer spooler serv.
 ;
 PRTASK	LD	B,PRMAX ;max chars out / interrupt
 PRAGAIN	CALL	PRIN	;read printer status
 	RET	NZ	;not ready
 	LD	A,PRDCB	;printer DCB number
 	CALL	LOCDCB	;find the DCB
 	LD	E,(IX+15) ;get spooler counter
 	LD	D,(IX+16)
 	LD	A,D
 	OR	E
 	RET	Z
 	DEC	DE	;else decrement for char out
 	LD	(IX+15),E
 	LD	(IX+16),D
 	LD	L,(IX+11) ;buffer start
 	LD	H,(IX+12)
 	LD	E,(IX+19) ;take offset
 	LD	D,(IX+20)
 	ADD	HL,DE	;point to char
 	LD	A,(HL)	;get it
 	CALL	PRINC	;increment pointer
 	LD	(IX+19),E ;update DCB offset
 	LD	(IX+20),D
 	CALL	PROUT	;send the character
 	DJNZ	PRAGAIN ;go for max chars
 	RET
 ;
 ; user timer service
 ;
 TMRTASK	LD	HL,FLAGK1 ;point to flag byte
 	BIT	3,(HL)	;timer on?
 	RET	Z	;nope
 	LD	HL,DATAK1 ;point to counter
 	DEC	(HL)	;time yet?
 	RET	NZ	;not time
 	LD	HL,FLAGK1 ;flag again
 	RES	3,(HL)	;turn it off
 	LD	HL,(DATAK2) ;get vector
 	JP	(HL)	;go!
 ;
 ; break key processor
 ;
 BRKTASK	LD	HL,FLAGK1 ;flag byte
 	BIT	6,(HL)	;break in buffer
 	RET	Z	;nope
 	RES	6,(HL)	;reset it
 	BIT	2,(HL)	;debug on?
 	JR	NZ,BRKDBG ;go if yes
 	BIT	7,(HL)	;user break vector
 	RET	Z	;nope
 	LD	HL,(DATAK6) ;get vector
 	JP	(HL)	;go!
 BRKDBG	SET	1,(HL)	;set debug GO
 	RET
 ;
 ; HOLD key processor
 ;
 HLDTASK	LD	HL,FLAGK1 ;flag
 	BIT	4,(HL)	;hold in buffer
 	RET	Z	;nope
 	RES	4,(HL)	;turn it off
 	BIT	5,(HL)	;vectored?
 	RET	Z	;nope
 	LD	HL,(DATAK6) ;get vector
 	JP	(HL)	;go!
 ;
 ; normal vector for BREAK processor
 ;
 BRKVEC	RST	0	;back to DOS
 ;
 ; normal vector for HOLD and TIMER vectors
 ;
 HLDVEC	EQU	$
 TMRVEC	EQU	$
 	RET		;nil for now
 ;
 ; clock processor
 ;
 CLKTASK	LD	HL,FLAGK2 ;flag
 	BIT	7,(HL)	;on?
 	RET	Z
 	LD	A,DDCB	;video DCB number
 	CALL	LOCDCB	;find it
 	LD	A,(IX+5) ;get video mask byte
 	PUSH	AF	;save mask
 	OR	80H	;video enable
 	OUT	(0FFH),A ;turn it on
 	LD	HL,0F800H+69 ;clock display position
 	LD	DE,TIME$+3 ;point to hour
 	LD	B,3	;3 loops
 	JR	CLK2
 CLK1	LD	(HL),':' ;time separator
 	INC	HL	;next video slot
 CLK2	LD	A,(DE)	;get time byte
 	DEC	DE	;point to next
 	CALL	CLKASC	;quick ascii
 	LD	(HL),C	;put MSB
 	INC	HL
 	LD	(HL),A	;put LSB ascii
 	INC	HL	;next
 	DJNZ	CLK1	;more?
 	POP	AF	;get video mask back
 	OUT	(0FFH),A ;put it back like it was
 	RET		;done
 CLKASC	LD	C,'0'	;start for MSB
 CLKA1	SUB	10	;less 10
 	JR	C,CLKA2
 	INC	C
 	JR	CLKA1
 CLKA2	ADD	A,3AH	;make it ascii
 	RET
 ;
 ; trace processor
 ;
 TRCTASK	LD	HL,FLAGK2
 	BIT	6,(HL)	;on?
 	RET	Z	;nope
 	LD	A,DDCB	;video DCB number
 	CALL	LOCDCB	;find it
 	LD	A,(IX+5) ;get video mask
 	PUSH	AF
 	OR	80H	;set 7
 	OUT	(0FFH),A ;enable video ram
 	LD	HL,0F800H+64 ;video location
 	LD	DE,0	;get trace address
 TRADD	EQU	$-2	;set from NMI service
 	LD	A,D	;get MSB
 	CALL	TRHEX	;quick hex convert
 	LD	A,E	;get LSB
 	CALL	TRHEX	;put on video
 	POP	AF	;restore video mask
 	OUT	(0FFH),A ;put it back
 	RET		;done
 TRHEX	LD	C,A	;save byte
 	RRCA		;move top bits down
 	RRCA
 	RRCA
 	RRCA
 	AND	0FH	;mask it off
 	CALL	TRHEXA	;make it ascii
 	LD	(HL),A	;MSB to video
 	INC	HL
 	LD	A,C
 	AND	0FH	;low 4 bits
 	CALL	TRHEXA	;make it ascii
 	LD	(HL),A	;LSB to video
 	INC	HL
 	RET		;done
 TRHEXA	ADD	A,'0'	;add ascii to it
 	CP	3AH	;0-9?
 	RET	C
 	ADD	A,7	;adjust for A-F
 	RET
 ;
 ; alive processor
 ;
 ALVTASK	LD	HL,FLAGK2 ;flag byte
 	BIT	5,(HL)	;on?
 	RET	Z	;nope
 	LD	A,DDCB	;video DCB number
 	CALL	LOCDCB	;find it
 	LD	A,(IX+5) ;get video mask
 	PUSH	AF	;save it
 	OR	80H	;enable video ram
 	OUT	(0FFH),A
 	LD	HL,0F800H+78
 	LD	E,99H
 	LD	BC,9A9EH
 	CALL	ALIVE	;put on video
 	INC	L	;next video
 	LD	E,9CH
 	LD	BC,9D9EH
 	CALL	ALIVE	;to video
 	POP	AF	;get video
 	OUT	(0FFH),A ;put it back
 	RET		;done
 ALIVE	LD	A,(HL)	;get the current byte
 	CP	E	;this one?
 	JR	Z,ALIV1	;go if yes
 	CP	B	;this one?
 	JR	Z,ALIV2	;go if yes
 	LD	(HL),E	;reset it
 	RET
 ALIV1	LD	(HL),B
 	RET
 ALIV2	LD	(HL),C
 	RET
 *EJECT
 ;
 	END
